/*
 * MAX : sync1.cc, Wed Jan  2 14:44:59 CET 2002 -sg
 *
 * Device driver for SYNC1_1
 * channel 1 of 2 channels
 *
 * M. Dammasch  15.12.1998 error()-handling changed (only 0 returns)
 * M. Dammasch  03.06.1999 define uniqie DEBUG-flag for verbose/silent operation
 * 
 * This file is part of Max data acquisition software
 * (c) 1997-98 by D. Rohloff & C. Otto & M. Dammasch
 *
 * Max is free software; you can redistribute it and/or modify it
 * under the terms of the version 2 of GNU General Public License as
 * published by the Free Software Foundation.
 *
 * Max is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License
 * (see the file COPYING) in this directory; if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <math.h>
#include "list.hh"
#include "sched.hh"
#include "error.hh"

extern "C" {
#include "camacs.h"
}

#define CH1_REG  0
#define CH2_REG  1

#define  CH1_DELAY_REG  0
#define  CH2_DELAY_REG  2
#define  CH1_HOLD_REG   1
#define  CH2_HOLD_REG   3
#define  CH1_CTRL_REG   0
#define  CH2_CTRL_REG	1

#undef DEBUG_DEV_SYNC1

/**
	global variables
**/
static int    N, TLAM;	 	      // Station, global timeout
static double delay_1;        // Delay value (ch1) - variable
static int    delay_2;	      // Delay value (ch2) - fix
static double delay_step_1;	      // delay stepsize (ch1)
static int    hold_1, hold_2;	      // Hold value
static int    module_on_1, module_on_2;	      // controlreg. bit #1
static int    trigger_ext_1, trigger_ext_2;       // controlreg. bit #2
static int    trigger_gated_1, trigger_gated_2;     // controlreg. bit #3

/**
	device functions for sync1_1 (sync1 ch1 only)
**/
int sync1_1_init(List *device_data, Scheduler *parameters)
{
  unsigned int dummy=0;	

  printf("\nsync1_1: initializing... ");

  // init CAMAC
  if (camac_init() != 1)
    if (!parameters->error(E_CAMAC,"Couldn't initialize device sync1_1 "))
      return 0;

  device_data->getvar(&TLAM,"all","timeout");  // get general timeout value
  device_data->getvar(&N,"sync1","N");         // get number of CAMAC slot

  camac(25,CH1_REG,N,&dummy);  // Reset Module Channel 1

  device_data->getvar(&delay_1,"sync1_1","delay");  // get delay parameter
  device_data->getvar(&delay_step_1,"sync1_1","delay_step");  // get delay stepsize
  dummy = (unsigned int) delay_1;
  camac(16,CH1_DELAY_REG,N,&dummy);  // Set Delay register (ch1)

  device_data->getvar(&hold_1,"sync1_1","hold");  // get hold  parameter
  dummy = (unsigned int) hold_1;
  camac(16,CH1_HOLD_REG,N,&dummy);  // set Hold register (ch1)

  device_data->getvar(&module_on_1,"sync1_1","module_on");
  device_data->getvar(&trigger_ext_1,"sync1_1","trigger_ext");
  device_data->getvar(&trigger_gated_1,"sync1_1","trigger_gated");  // get control parameter
  dummy = (unsigned int) trigger_gated_1*4+trigger_ext_1*2+module_on_1;
  camac(17,CH1_CTRL_REG,N,&dummy);  // Set Control register (ch1)

  printf("done ");
  return 1;
}


int sync1_1_close(List *device_data, Scheduler *parameters)
{
  unsigned int dummy=0;	

  camac(25,CH1_REG,N,&dummy);  		// reset module (ch1)
  camac(17,CH1_CTRL_REG,N,&dummy);  	// reset control reg. (ch1)
  printf("\nclosing sync1_1 ");

  if (camac_close() != 1)
    if (!parameters->error(E_CAMAC,"Couldn't close device sync1_1 "))
      return 0;

  return 1;
}


// pre-trigger device function for sync1_1:
// increase delay time, if required
int sync1_1_pre(List *device_data, Scheduler *parameters)
{
  unsigned int dummy, qx=0;

  if (delay_step_1 > 0.0) {
    delay_1 += delay_step_1;
    dummy = (unsigned int)delay_1;
    camac(16,CH1_DELAY_REG,N,&dummy);  // Set Delay register (ch1)

    camac(26,CH1_REG,N,&dummy);  // enable LAM (ch1)

    if (!((qx = camac(27,CH1_REG,N,&dummy)) & 2)) // test, if really enabled
      if (!parameters->error(E_CAMAC,"sync1_1: LAM not enabled"))
	return 0;

#ifdef DEBUG_DEV_SYNC1
    printf("\nsync1_1: LAM enabled");
#endif
  }

  return 1;
}

// main device function for sync1_1
int sync1_1(List *device_data, Scheduler *parameters)
{
  int          i;
  unsigned int qx, dummy=0;

  if (delay_step_1 > 0.0)
  {
    i = 0;
    qx = camac(8,CH1_REG,N,&dummy);
    while (++i < TLAM && !(qx & 2))
    {
      qx = camac(8,CH1_REG,N,&dummy);
    }
#ifdef DEBUG_DEV_SYNC1
    printf("\nsync1_1: LAM %s (%d - %d/%d)",((qx&2)?("occured"):("missing")),qx,i,TLAM);
#endif

    if (i >= TLAM)
      if (!parameters->error(E_TIMEOUT,"sync1_1: timed out"))
	return 0;

    camac(10,CH1_REG,N,&dummy);  // erase LAM (ch1)
    camac(24,CH1_REG,N,&dummy);  // disable LAM (ch1)
  }

  return 1;
} /* of SYNC1_1 */

/**
	device functions for sync1_2 (sync1 ch2 only)
**/
int sync1_2_init(List *device_data, Scheduler *parameters)
{
  unsigned int dummy=0;	

  printf("\nsync1_2: initializing... ");

  //init CAMAC
  if (camac_init() != 1)
    if (!parameters->error(E_CAMAC,"Couldn't initialize device sync1_2 "))
      return 0;

  device_data->getvar(&TLAM,"all","timeout"); 	                //get general timeout value
  device_data->getvar(&N,"sync1","N"); 	                        //get number of CAMAC station

  camac(25,CH2_REG,N,&dummy);  /* Reset Module Channel 1 */

  device_data->getvar(&delay_2,"sync1_2","delay"); 	                //get delay parameter
  dummy = (unsigned int) delay_2;
  camac(16,CH2_DELAY_REG,N,&dummy);  // Set Delay register (ch2)

  device_data->getvar(&hold_2,"sync1_2","hold"); 	                //get hold  parameter
  dummy = (unsigned int) hold_2;
  camac(16,CH2_HOLD_REG,N,&dummy);  // Set Hold register (ch2)

  device_data->getvar(&module_on_2,"sync1_2","module_on");
  device_data->getvar(&trigger_ext_2,"sync1_2","trigger_ext");
  device_data->getvar(&trigger_gated_2,"sync1_2","trigger_gated");  //get control parameter
  dummy = (unsigned int) trigger_gated_2*4+trigger_ext_2*2+module_on_2;
  camac(17,CH2_CTRL_REG,N,&dummy);  // Set Control register (ch2)

  printf("done ");
  return 1;
}

int sync1_2_close(List *device_data, Scheduler *parameters)
{
  unsigned int dummy=0;	

  camac(25,CH2_REG,N,&dummy);  		// reset module (ch2)
  camac(17,CH2_CTRL_REG,N,&dummy);  	// reset control reg. (ch2)
  printf("\nclosing sync1_2");

  if (camac_close() != 1)
    if (!parameters->error(E_CAMAC,"Couldn't close device sync1_2 "))
      return 0;

  return 1;
}

// pre-trigger device function for sync1_2
int sync1_2_pre(List *device_data, Scheduler *parameters)
{
  return 1;
}

// main device function for sync1_2
int sync1_2(List *device_data, Scheduler *parameters)
{
  return 1;
} /* of SYNC1_2 */
















