/*
 * MAX : database.h, Sam Dez 29 19:34:06 CET 2001 -sg
 *
 * This file is part of Max data acquisition software
 * Copyright (C) 1998 Christian Rosen
 *
 * Max is free software; you can redistribute it and/or modify it
 * under the terms of the version 2 of GNU General Public License as
 * published by the Free Software Foundation.
 *
 * Max is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License
 * (see the file COPYING) in this directory; if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


//header file of database class
//this class stores all measured data received by the MainGUI
//and saves them to disk

#ifndef DATABASE_H
#define DATABASE_H

#define BACKUP_ARRAY_SIZE MAX_ARRAY_SIZE //size for array to be plotted, rest is truncated
	   			
#include <assert.h>
#include <stdarg.h>
#include <string.h>
#include <stdio.h>

//STL includes

#include <vector.h>
#include <list.h>

//lokal includes

#include "qtincludes.h"
#include "error.h"
#include "data.h"
#include "priority.h"
//
//typelist of measured data for save-routine
//structures needed for save routine
//to determine the variable type for storage to disk
//
typedef struct VarType {
  int       type;            //1 = int, 2 = double
  };
typedef struct ArrayType {
  int       size;
  int       type;             //1 = int, 2 = double
};
//
//class definitions for the data base containing the measured data
//the constructor is overloaded, hence different data types can be stored there
//
//Image class -> store linked list of images
//
class ImageListElement {
  friend class Database;
public:
  QImage Image(void) const { return image; }
private :
  ImageListElement(QImage *i,ImageListElement *I = 0) {
    image = i->copy();
    next  = I;
  }
  ~ImageListElement() {// delete image;
  }
  QImage           image;
  ImageListElement *next;
};
//
//base class for scalar data (integers and doubles)
//measured data is stored in a vector of this class
//this is faster than a linked list
//this class provides several default constructors for the different data types
//see : Dlist in database.c
//
class Scalar_Vector_Element {
  friend class Database;
  public  :
   ~Scalar_Vector_Element() {   }
   Scalar_Vector_Element(int x )     { i_var = x;type =1; }
   Scalar_Vector_Element(double x)   { d_var = x;type =2; }
  private :
   int    type;       //1 = int, 2 = double
   int    i_var;
   double d_var;
};
//
//base class for linked list of vectors
//this class is used to store the array of int and double in a vector
//see :  Alist in database.c
//
class Array_Vector_Element {
 friend class Database;
  public  :
  ~Array_Vector_Element() {
    //i_vec.~vector<int>   i_vec ();    //destroy it
    //d_vec.~vector<double>d_vec (); //destroy it
  }
  Array_Vector_Element(int x[],size_t size) {
    type = 1;
    i_vec.reserve(size);
    for (unsigned int i=0; i< size; i++) i_vec.push_back(x[i]);
    //   d_vec = 0;
  }
  Array_Vector_Element(double x[],size_t size) {
    type = 2;
    d_vec.reserve(size);                                 //reserve size
    for (unsigned int i=0; i< size; i++) d_vec.push_back(x[i]); //copy elements
    //i_vec = 0;
  }
  private :
  int            type; //1 = int, 2 = double
  vector<int>    i_vec;
  vector<double> d_vec;
};

/****************************************************************************/
/* main header part */
/****************************************************************************/
class Database  {

public :          //initialize to normal mode with external trigger(=0)
  Database();
  ~Database();
  void init();   //!initialize database and clear all variables
  int  maximal_integrations  (void );  //return max_integ al channel number
  //
  //class functions for database
  //
  void add_data( int      );
  void add_data( double   );
  void add_data( char     );
  void add_data( char *   );
  void add_data( QImage * );
  vector<int>    * add_data( int*,int );     //add array of int containing  n elements to ArrayList
  vector<double> * add_data( double *, int ); //add integer array, return address of it
  void store   ( int      );
  void store   ( double   );
  void store   ( int *,int);
  void store   ( char *   );     //filename for image loading from disk
  //
  //functions for internal data handling
  //
  void turn_around(void );
  void arrayList_turn_around();
  void imageList_turn_around();
  void remove       ( void );        //!removes complete list out of mem
  void remove       ( int  );        //!remove n elements from the list, start at the beginning
  void remove_1st   ( void );        //!removes 1st Data_element out of list
  void array_remove_1st( void );     //!remove 1st element out of array list
  void array_remove  (int  );        //!remove n elements from array database, start at the beginning
  void printlist    ( void );        //just for debugging purposes
  void save_data   (const char *, VarType *,int);   //new save routine
  void save_arrays  ( const char *, ArrayType *); //save integer strings
  void save_images  ( const char *,int ,char *);
  void set_N_Elements (int );       //!Set the number of Data Elements per channel in database
  void set_N_Arrays(int);	    //!Set the number of Arrays per channel in the database
  void set_MonMode(bool );          //!set monitor mode flag TRUE = on, FALSE = OFF
  int  length_of_list( void );
  int  length_of_arraylist( void );
  int  length_of_imagelist( void );
  int  maxarraylength();             //find length of longest array
  int  array_empty   ( void );       //return NULL if array is empty
  int  database_empty( void );       //return NULL if no data is saved
  int  image_empty   ( void );
  ImageListElement *end_of_image_list();
private :
//List of measured data packets see : data.cc
  //this class also contains functions to convert data to packets
  PacketList         *Que;
  dataPacket         dat_packet;  //Packet for sending measured data  -> see packet.h
  int max_channels;
  int max_integrations;
  int channels;                 //current channel
  int integrations;             //maximum number of integrations per cycle
  int priority;                 //actual priority currently called
  int stop ;
  int s_mode;                   //False if no signals for display are called from scheduler (high speed)
  int t_mode;                   //TRUE = external trigger ,FALSE = internal
  int device_status;            //return status of devices -> (used by SLOT_measure only)
  uint period;                  //delay for internal trigger contains ms
  bool busy;                    //busy flag to block parallel measurements
  //
  //elements for database functions
  //
  QList <Scalar_Vector_Element> Dlist; //!linked list of the Scalar data in a class
  QList <Array_Vector_Element>  Alist; //!linked list of vector data in a class of vectors
  vector<VarType> *Sc_Data;
  ImageListElement *Image;      //Pointer to element of image list
  //
  //
  char              *typelist;
  int               var_index;         //index off all "stored" variables
  int               temp_var_index;
  int               element_counter;   //contains the numberof measured data stored by device functions
  int               array_counter;     //counter for array-backup array
  int               temp_counter;      //temp counter for add_data function
  int               array_temp_counter;//temp counter for array add_data function
  int               maxarraysize;
  int 		    varcounter;        //!number of elements per channel stored in the list
  int		    arraycounter;      //!number of arrays per channel stored in the lis
  bool   	    monitor_mode;
  int 		    *z;
  double 	    *zd;
  //end of database variables
};

#endif
//don't write  behind the endif
